@@ -34,7 +34,6 @@
 #include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
 
-#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-contact-groups.h>
@@ -763,11 +762,9 @@ contact_list_view_drag_data_get (GtkWidget        *widget,
 	g_object_unref (contact);
 	str = g_strconcat (account_id, ":", contact_id, NULL);
 
-	switch (info) {
-	case DND_DRAG_TYPE_CONTACT_ID:
+	if (info == DND_DRAG_TYPE_CONTACT_ID) {
 		gtk_selection_data_set (selection, drag_atoms_source[info], 8,
 					(guchar *) str, strlen (str) + 1);
-		break;
 	}
 
 	g_free (str);
@@ -826,8 +823,6 @@ contact_list_view_popup_menu_idle_cb (gpointer user_data)
 		gtk_menu_popup (GTK_MENU (menu),
 				NULL, NULL, NULL, NULL,
 				data->button, data->time);
-		g_object_ref_sink (menu);
-		g_object_unref (menu);
 	}
 
 	g_slice_free (MenuPopupData, data);
@@ -894,7 +889,7 @@ contact_list_view_row_activated (GtkTreeView       *view,
 	if (contact) {
 		DEBUG ("Starting a chat");
 		empathy_dispatcher_chat_with_contact (contact,
-			gtk_get_current_event_time (), NULL, NULL);
+			gtk_get_current_event_time ());
 		g_object_unref (contact);
 	}
 }
