@@ -27,13 +27,11 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <telepathy-glib/util.h>
-#include <telepathy-logger/log-manager.h>
+
 #include <folks/folks.h>
 #include <folks/folks-telepathy.h>
 
-#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-dispatcher.h>
-#include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-individual-manager.h>
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-utils.h>
@@ -42,11 +40,34 @@
 #include "empathy-images.h"
 #include "empathy-log-window.h"
 #include "empathy-contact-dialogs.h"
-#include "empathy-individual-dialogs.h"
+#include "empathy-gtk-enum-types.h"
+#include "empathy-individual-edit-dialog.h"
 #include "empathy-individual-information-dialog.h"
 #include "empathy-ui-utils.h"
 #include "empathy-share-my-desktop.h"
 #include "empathy-linking-dialog.h"
+#include "empathy-call-utils.h"
+
+#define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyIndividualMenu)
+
+typedef struct {
+  FolksIndividual *individual; /* owned */
+  EmpathyIndividualFeatureFlags features;
+} EmpathyIndividualMenuPriv;
+
+enum {
+  PROP_INDIVIDUAL = 1,
+  PROP_FEATURES,
+};
+
+enum {
+  SIGNAL_LINK_CONTACTS_ACTIVATED,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
+G_DEFINE_TYPE (EmpathyIndividualMenu, empathy_individual_menu, GTK_TYPE_MENU);
 
 static void
 individual_menu_add_personas (GtkMenuShell *menu,
@@ -67,7 +88,7 @@ individual_menu_add_personas (GtkMenuShell *menu,
    * functionality */
   for (l = personas; l != NULL; l = l->next)
     {
-      if (!TPF_IS_PERSONA (l->data))
+      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
         continue;
 
       persona_count++;
@@ -96,7 +117,7 @@ individual_menu_add_personas (GtkMenuShell *menu,
       const gchar *account;
       GtkWidget *action;
 
-      if (!TPF_IS_PERSONA (persona))
+      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
         continue;
 
       tp_contact = tpf_persona_get_contact (persona);
@@ -104,10 +125,16 @@ individual_menu_add_personas (GtkMenuShell *menu,
 
       store = folks_persona_get_store (FOLKS_PERSONA (persona));
       account = folks_persona_store_get_display_name (store);
+
+      /* Translators: this is used in the context menu for a contact. The first
+       * parameter is a contact ID (e.g. foo@jabber.org) and the second is one
+       * of the user's account IDs (e.g. me@hotmail.com). */
       label = g_strdup_printf (_("%s (%s)"),
           folks_persona_get_display_id (FOLKS_PERSONA (persona)), account);
 
       contact_item = gtk_image_menu_item_new_with_label (label);
+      gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (contact_item),
+                                                 TRUE);
       contact_submenu = gtk_menu_new ();
       gtk_menu_item_set_submenu (GTK_MENU_ITEM (contact_item), contact_submenu);
       image = gtk_image_new_from_icon_name (
@@ -168,29 +195,38 @@ individual_menu_add_personas (GtkMenuShell *menu,
     }
 }
 
-GtkWidget *
-empathy_individual_menu_new (FolksIndividual *individual,
-    EmpathyIndividualFeatureFlags features)
+static void
+individual_link_menu_item_activate_cb (EmpathyIndividualMenu *self)
 {
-  GtkWidget *menu;
+  EmpathyIndividualMenuPriv *priv = GET_PRIV (self);
+  GtkWidget *dialog;
+
+  dialog = empathy_linking_dialog_show (priv->individual, NULL);
+  g_signal_emit (self, signals[SIGNAL_LINK_CONTACTS_ACTIVATED], 0, dialog);
+}
+
+static void
+empathy_individual_menu_init (EmpathyIndividualMenu *self)
+{
+  EmpathyIndividualMenuPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      EMPATHY_TYPE_INDIVIDUAL_MENU, EmpathyIndividualMenuPriv);
+
+  self->priv = priv;
+}
+
+static void
+constructed (GObject *object)
+{
+  EmpathyIndividualMenuPriv *priv = GET_PRIV (object);
   GtkMenuShell *shell;
   GtkWidget *item;
+  FolksIndividual *individual;
+  EmpathyIndividualFeatureFlags features;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
-
-  if (features == EMPATHY_INDIVIDUAL_FEATURE_NONE)
-    return NULL;
-
-  menu = gtk_menu_new ();
-  shell = GTK_MENU_SHELL (menu);
-
-  /* Add Contact */
-  item = empathy_individual_add_menu_item_new (individual);
-  if (item)
-    {
-      gtk_menu_shell_append (shell, item);
-      gtk_widget_show (item);
-    }
+  /* Build the menu */
+  shell = GTK_MENU_SHELL (object);
+  individual = priv->individual;
+  features = priv->features;
 
   /* Chat */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_CHAT)
@@ -242,7 +278,7 @@ empathy_individual_menu_new (FolksIndividual *individual,
   gtk_widget_show (item);
 
   /* Menu items to target specific contacts */
-  individual_menu_add_personas (GTK_MENU_SHELL (menu), individual, features);
+  individual_menu_add_personas (GTK_MENU_SHELL (object), individual, features);
 
   /* Separator */
   if (features & (EMPATHY_INDIVIDUAL_FEATURE_EDIT |
@@ -268,6 +304,10 @@ empathy_individual_menu_new (FolksIndividual *individual,
     {
       item = empathy_individual_link_menu_item_new (individual);
       gtk_menu_shell_append (shell, item);
+
+      g_signal_connect_swapped (item, "activate",
+          (GCallback) individual_link_menu_item_activate_cb, object);
+
       gtk_widget_show (item);
     }
 
@@ -286,154 +326,170 @@ empathy_individual_menu_new (FolksIndividual *individual,
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
     }
-
-  return menu;
 }
 
 static void
-empathy_individual_add_menu_item_activated (GtkMenuItem *item,
-  FolksIndividual *individual)
+get_property (GObject *object,
+    guint param_id,
+    GValue *value,
+    GParamSpec *pspec)
 {
-  GtkWidget *toplevel;
+  EmpathyIndividualMenuPriv *priv;
 
-  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (item));
-  if (!gtk_widget_is_toplevel (toplevel) || !GTK_IS_WINDOW (toplevel))
-    toplevel = NULL;
+  priv = GET_PRIV (object);
 
-  empathy_new_individual_dialog_show_with_individual (GTK_WINDOW (toplevel),
-      individual);
+  switch (param_id)
+    {
+      case PROP_INDIVIDUAL:
+        g_value_set_object (value, priv->individual);
+        break;
+      case PROP_FEATURES:
+        g_value_set_flags (value, priv->features);
+        break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+        break;
+    }
+}
+
+static void
+set_property (GObject *object,
+    guint param_id,
+    const GValue *value,
+    GParamSpec *pspec)
+{
+  EmpathyIndividualMenuPriv *priv;
+
+  priv = GET_PRIV (object);
+
+  switch (param_id)
+    {
+      case PROP_INDIVIDUAL:
+        priv->individual = g_value_dup_object (value);
+        break;
+      case PROP_FEATURES:
+        priv->features = g_value_get_flags (value);
+        break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+        break;
+    }
+}
+
+static void
+dispose (GObject *object)
+{
+  EmpathyIndividualMenuPriv *priv = GET_PRIV (object);
+
+  tp_clear_object (&priv->individual);
+
+  G_OBJECT_CLASS (empathy_individual_menu_parent_class)->dispose (object);
+}
+
+static void
+empathy_individual_menu_class_init (EmpathyIndividualMenuClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->constructed = constructed;
+  object_class->get_property = get_property;
+  object_class->set_property = set_property;
+  object_class->dispose = dispose;
+
+  /**
+   * EmpathyIndividualMenu:individual:
+   *
+   * The #FolksIndividual the menu is for.
+   */
+  g_object_class_install_property (object_class, PROP_INDIVIDUAL,
+      g_param_spec_object ("individual",
+          "Individual",
+          "The #FolksIndividual the menu is for.",
+          FOLKS_TYPE_INDIVIDUAL,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * EmpathyIndividualMenu:features:
+   *
+   * A set of feature flags controlling which entries are shown.
+   */
+  g_object_class_install_property (object_class, PROP_FEATURES,
+      g_param_spec_flags ("features",
+          "Features",
+          "A set of feature flags controlling which entries are shown.",
+          EMPATHY_TYPE_INDIVIDUAL_FEATURE_FLAGS,
+          EMPATHY_INDIVIDUAL_FEATURE_NONE,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+
+  signals[SIGNAL_LINK_CONTACTS_ACTIVATED] =
+      g_signal_new ("link-contacts-activated", G_OBJECT_CLASS_TYPE (klass),
+          G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+          g_cclosure_marshal_VOID__OBJECT,
+          G_TYPE_NONE, 1, EMPATHY_TYPE_LINKING_DIALOG);
+
+  g_type_class_add_private (object_class, sizeof (EmpathyIndividualMenuPriv));
 }
 
 GtkWidget *
-empathy_individual_add_menu_item_new (FolksIndividual *individual)
+empathy_individual_menu_new (FolksIndividual *individual,
+    EmpathyIndividualFeatureFlags features)
 {
-  GtkWidget *item;
-  GtkWidget *image;
-  EmpathyIndividualManager *manager = NULL;
-  EmpathyContact *contact = NULL;
-  TpConnection *connection;
-  GList *l, *members;
-  gboolean found = FALSE;
-  EmpathyIndividualManagerFlags flags;
-
   g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
+  g_return_val_if_fail (features != EMPATHY_INDIVIDUAL_FEATURE_NONE, NULL);
 
-  if (!empathy_individual_manager_initialized ())
-    {
-      item = NULL;
-      goto out;
-    }
-
-  manager = empathy_individual_manager_dup_singleton ();
-  contact = empathy_contact_dup_from_folks_individual (individual);
-  connection = empathy_contact_get_connection (contact);
-
-  flags = empathy_individual_manager_get_flags_for_connection (manager,
-      connection);
-
-  if (!(flags & EMPATHY_INDIVIDUAL_MANAGER_CAN_ADD))
-    {
-      item = NULL;
-      goto out;
-    }
-
-  members = empathy_individual_manager_get_members (
-      EMPATHY_INDIVIDUAL_MANAGER (manager));
-
-  for (l = members; l && !found; l = l->next)
-    {
-      if (!tp_strdiff (folks_individual_get_id (l->data),
-              folks_individual_get_id (individual)))
-        {
-          found = TRUE;
-        }
-    }
-  g_list_free (members);
-
-  if (found)
-    {
-      item = NULL;
-      goto out;
-    }
-
-  item = gtk_image_menu_item_new_with_mnemonic (_("_Add Contact…"));
-  image = gtk_image_new_from_icon_name (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-
-  g_signal_connect (item, "activate",
-      G_CALLBACK (empathy_individual_add_menu_item_activated),
-      individual);
-
-out:
-  tp_clear_object (&contact);
-  tp_clear_object (&manager);
-
-  return item;
+  return g_object_new (EMPATHY_TYPE_INDIVIDUAL_MENU,
+      "individual", individual,
+      "features", features,
+      NULL);
 }
 
-typedef gboolean (*SensitivityPredicate) (EmpathyContact *contact);
-
 /* Like menu_item_set_first_contact(), but always operates upon the given
- * contact */
+ * contact. If the contact is non-NULL, it is assumed that the menu entry should
+ * be sensitive. */
 static gboolean
 menu_item_set_contact (GtkWidget *item,
     EmpathyContact *contact,
     GCallback activate_callback,
-    SensitivityPredicate sensitivity_predicate)
+    EmpathyActionType action_type)
 {
-  gboolean contact_valid = TRUE;
+  gboolean can_do_action = FALSE;
 
-  if (sensitivity_predicate != NULL)
+  if (contact != NULL)
+    can_do_action = empathy_contact_can_do_action (contact, action_type);
+  gtk_widget_set_sensitive (item, can_do_action);
+
+  if (can_do_action == TRUE)
     {
-      contact_valid = sensitivity_predicate (contact);
-      gtk_widget_set_sensitive (item, sensitivity_predicate (contact));
+      /* We want to make sure that the EmpathyContact stays alive while the
+       * signal is connected. */
+      g_signal_connect_data (item, "activate", G_CALLBACK (activate_callback),
+          g_object_ref (contact), (GClosureNotify) g_object_unref, 0);
     }
 
-  g_signal_connect (item, "activate", G_CALLBACK (activate_callback),
-      contact);
-
-  return contact_valid;
+  return can_do_action;
 }
 
 /**
- * Set the given menu @item to call @activate_callback upon the first valid
- * TpContact associated with @individual whenever @item is activated.
+ * Set the given menu @item to call @activate_callback using the TpContact
+ * (associated with @individual) with the highest availability who is also valid
+ * whenever @item is activated.
  *
- * @sensitivity_predicate is an optional function to determine whether the menu
- * item should be insensitive (if the function returns @FALSE). Otherwise, the
- * menu item's sensitivity will not change.
+ * @action_type is the type of action performed by the menu entry; this is used
+ * so that only contacts which can perform that action (e.g. are capable of
+ * receiving video calls) are selected, as appropriate.
  */
 static GtkWidget *
 menu_item_set_first_contact (GtkWidget *item,
     FolksIndividual *individual,
     GCallback activate_callback,
-    SensitivityPredicate sensitivity_predicate)
+    EmpathyActionType action_type)
 {
-  GList *personas, *l;
+  EmpathyContact *best_contact;
 
-  personas = folks_individual_get_personas (individual);
-  for (l = personas; l != NULL; l = l->next)
-    {
-      TpContact *tp_contact;
-      EmpathyContact *contact;
-      TpfPersona *persona = l->data;
-      gboolean contact_valid = TRUE;
-
-      if (!TPF_IS_PERSONA (persona))
-        continue;
-
-      tp_contact = tpf_persona_get_contact (persona);
-      contact = empathy_contact_dup_from_tp_contact (tp_contact);
-
-      contact_valid = menu_item_set_contact (item, contact,
-          G_CALLBACK (activate_callback), sensitivity_predicate);
-
-      g_object_unref (contact);
-
-      /* stop after the first valid match */
-      if (contact_valid)
-        break;
-    }
+  best_contact = empathy_contact_dup_best_for_action (individual, action_type);
+  menu_item_set_contact (item, best_contact, G_CALLBACK (activate_callback),
+      action_type);
+  tp_clear_object (&best_contact);
 
   return item;
 }
@@ -468,12 +524,14 @@ empathy_individual_chat_menu_item_new (FolksIndividual *individual,
   if (contact != NULL)
     {
       menu_item_set_contact (item, contact,
-          G_CALLBACK (empathy_individual_chat_menu_item_activated), NULL);
+          G_CALLBACK (empathy_individual_chat_menu_item_activated),
+          EMPATHY_ACTION_CHAT);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
-          G_CALLBACK (empathy_individual_chat_menu_item_activated), NULL);
+          G_CALLBACK (empathy_individual_chat_menu_item_activated),
+          EMPATHY_ACTION_CHAT);
     }
 
   return item;
@@ -485,8 +543,10 @@ empathy_individual_audio_call_menu_item_activated (GtkMenuItem *item,
 {
   g_return_if_fail (EMPATHY_IS_CONTACT (contact));
 
-  empathy_call_factory_new_call_with_streams (contact, TRUE, FALSE,
-      gtk_get_current_event_time (), NULL);
+  empathy_call_new_with_streams (empathy_contact_get_id (contact),
+      empathy_contact_get_account (contact),
+      TRUE, FALSE,
+      gtk_get_current_event_time ());
 }
 
 GtkWidget *
@@ -509,13 +569,13 @@ empathy_individual_audio_call_menu_item_new (FolksIndividual *individual,
     {
       menu_item_set_contact (item, contact,
           G_CALLBACK (empathy_individual_audio_call_menu_item_activated),
-          empathy_contact_can_voip_audio);
+          EMPATHY_ACTION_AUDIO_CALL);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
           G_CALLBACK (empathy_individual_audio_call_menu_item_activated),
-          empathy_contact_can_voip_audio);
+          EMPATHY_ACTION_AUDIO_CALL);
     }
 
   return item;
@@ -527,8 +587,10 @@ empathy_individual_video_call_menu_item_activated (GtkMenuItem *item,
 {
   g_return_if_fail (EMPATHY_IS_CONTACT (contact));
 
-  empathy_call_factory_new_call_with_streams (contact, TRUE, TRUE,
-      gtk_get_current_event_time (), NULL);
+  empathy_call_new_with_streams (empathy_contact_get_id (contact),
+      empathy_contact_get_account (contact),
+      TRUE, TRUE,
+      gtk_get_current_event_time ());
 }
 
 GtkWidget *
@@ -552,13 +614,13 @@ empathy_individual_video_call_menu_item_new (FolksIndividual *individual,
     {
       menu_item_set_contact (item, contact,
           G_CALLBACK (empathy_individual_video_call_menu_item_activated),
-          empathy_contact_can_voip_video);
+          EMPATHY_ACTION_VIDEO_CALL);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
           G_CALLBACK (empathy_individual_video_call_menu_item_activated),
-          empathy_contact_can_voip_video);
+          EMPATHY_ACTION_VIDEO_CALL);
     }
 
   return item;
@@ -574,21 +636,6 @@ empathy_individual_log_menu_item_activated (GtkMenuItem *item,
       empathy_contact_get_id (contact), FALSE, NULL);
 }
 
-static gboolean
-contact_has_log (EmpathyContact *contact)
-{
-  TplLogManager *manager;
-  gboolean have_log;
-
-  manager = tpl_log_manager_dup_singleton ();
-  have_log = tpl_log_manager_exists (manager,
-      empathy_contact_get_account (contact), empathy_contact_get_id (contact),
-      FALSE);
-  g_object_unref (manager);
-
-  return have_log;
-}
-
 GtkWidget *
 empathy_individual_log_menu_item_new (FolksIndividual *individual,
     EmpathyContact *contact)
@@ -609,13 +656,13 @@ empathy_individual_log_menu_item_new (FolksIndividual *individual,
     {
       menu_item_set_contact (item, contact,
           G_CALLBACK (empathy_individual_log_menu_item_activated),
-          contact_has_log);
+          EMPATHY_ACTION_VIEW_LOGS);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
           G_CALLBACK (empathy_individual_log_menu_item_activated),
-          contact_has_log);
+          EMPATHY_ACTION_VIEW_LOGS);
     }
 
   return item;
@@ -651,13 +698,13 @@ empathy_individual_file_transfer_menu_item_new (FolksIndividual *individual,
     {
       menu_item_set_contact (item, contact,
           G_CALLBACK (empathy_individual_file_transfer_menu_item_activated),
-          empathy_contact_can_send_files);
+          EMPATHY_ACTION_SEND_FILE);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
           G_CALLBACK (empathy_individual_file_transfer_menu_item_activated),
-          empathy_contact_can_send_files);
+          EMPATHY_ACTION_SEND_FILE);
     }
 
   return item;
@@ -692,13 +739,13 @@ empathy_individual_share_my_desktop_menu_item_new (FolksIndividual *individual,
     {
       menu_item_set_contact (item, contact,
           G_CALLBACK (empathy_individual_share_my_desktop_menu_item_activated),
-          empathy_contact_can_use_rfb_stream_tube);
+          EMPATHY_ACTION_SHARE_MY_DESKTOP);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
           G_CALLBACK (empathy_individual_share_my_desktop_menu_item_activated),
-          empathy_contact_can_use_rfb_stream_tube);
+          EMPATHY_ACTION_SHARE_MY_DESKTOP);
     }
 
   return item;
@@ -708,7 +755,7 @@ static void
 favourite_menu_item_toggled_cb (GtkCheckMenuItem *item,
   FolksIndividual *individual)
 {
-  folks_favourite_set_is_favourite (FOLKS_FAVOURITE (individual),
+  folks_favouritable_set_is_favourite (FOLKS_FAVOURITABLE (individual),
       gtk_check_menu_item_get_active (item));
 }
 
@@ -720,7 +767,7 @@ empathy_individual_favourite_menu_item_new (FolksIndividual *individual)
   item = gtk_check_menu_item_new_with_label (_("Favorite"));
 
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
-      folks_favourite_get_is_favourite (FOLKS_FAVOURITE (individual)));
+      folks_favouritable_get_is_favourite (FOLKS_FAVOURITABLE (individual)));
 
   g_signal_connect (item, "toggled",
       G_CALLBACK (favourite_menu_item_toggled_cb), individual);
@@ -760,12 +807,7 @@ empathy_individual_info_menu_item_new (FolksIndividual *individual)
 static void
 individual_edit_menu_item_activate_cb (FolksIndividual *individual)
 {
-  EmpathyContact *contact;
-
-  contact = empathy_contact_dup_from_folks_individual (individual);
-  empathy_contact_edit_dialog_show (contact, NULL);
-
-  tp_clear_object (&contact);
+  empathy_individual_edit_dialog_show (individual, NULL);
 }
 
 GtkWidget *
@@ -815,12 +857,6 @@ empathy_individual_edit_menu_item_new (FolksIndividual *individual)
   return item;
 }
 
-static void
-individual_link_menu_item_activate_cb (FolksIndividual *individual)
-{
-  empathy_linking_dialog_show (individual, NULL);
-}
-
 GtkWidget *
 empathy_individual_link_menu_item_new (FolksIndividual *individual)
 {
@@ -830,14 +866,18 @@ empathy_individual_link_menu_item_new (FolksIndividual *individual)
   g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
 
   item = gtk_image_menu_item_new_with_mnemonic (
-      C_("Link individual (contextual menu)", "_Link…"));
+      /* Translators: this is a verb meaning "to connect two contacts together
+       * to form a meta-contact". */
+      C_("Link individual (contextual menu)", "_Link Contacts…"));
   /* TODO */
   /*image = gtk_image_new_from_icon_name (GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
   gtk_widget_show (image);*/
 
-  g_signal_connect_swapped (item, "activate",
-      G_CALLBACK (individual_link_menu_item_activate_cb), individual);
+  /* Only allow trusted Individuals to be linked */
+  gtk_widget_set_sensitive (item,
+      folks_individual_get_trust_level (individual) ==
+          FOLKS_TRUST_LEVEL_PERSONAS);
 
   return item;
 }
@@ -905,7 +945,7 @@ room_sub_menu_activate_cb (GtkWidget *item,
           TpContact *tp_contact;
           GList *rooms;
 
-          if (!TPF_IS_PERSONA (persona))
+          if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
             continue;
 
           tp_contact = tpf_persona_get_contact (persona);
@@ -1005,7 +1045,7 @@ empathy_individual_invite_menu_item_new (FolksIndividual *individual,
           TpContact *tp_contact;
           EmpathyContact *contact_cur;
 
-          if (!TPF_IS_PERSONA (persona))
+          if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
             continue;
 
           tp_contact = tpf_persona_get_contact (persona);
