@@ -20,7 +20,8 @@
 
 /* empathy-account-assistant.c */
 
-#include <glib/gi18n.h>
+#include <config.h>
+#include <glib/gi18n-lib.h>
 #include <telepathy-glib/util.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -35,7 +36,6 @@
 #include <libempathy-gtk/empathy-account-widget.h>
 #include <libempathy-gtk/empathy-protocol-chooser.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
-#include <libempathy-gtk/empathy-conf.h>
 
 #define DEBUG_FLAG EMPATHY_DEBUG_ACCOUNT
 #include <libempathy/empathy-debug.h>
@@ -57,12 +57,12 @@ typedef enum {
   RESPONSE_CREATE_STOP = 2
 } CreateEnterPageResponse;
 
-enum {
+typedef enum {
   PAGE_INTRO = 0,
   PAGE_IMPORT = 1,
   PAGE_ENTER_CREATE = 2,
   PAGE_SALUT = 3,
-};
+} PageID;
 
 enum {
   PROP_PARENT = 1,
@@ -75,7 +75,7 @@ typedef struct {
   gboolean enter_create_forward;
   TpAccountManager *account_mgr;
   EmpathyConnectionManagers *connection_mgrs;
-  gint current_page_id;
+  PageID current_page_id;
 
   /* enter or create page */
   GtkWidget *enter_or_create_page;
@@ -319,7 +319,7 @@ account_assistant_apply_account_cb (GObject *source,
   EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (source);
   TpAccount *account;
 
-  empathy_account_settings_apply_finish (settings, result, &error);
+  empathy_account_settings_apply_finish (settings, result, NULL, &error);
 
   priv->is_creating = FALSE;
 
@@ -416,17 +416,25 @@ account_assistant_protocol_changed_cb (GtkComboBox *chooser,
 
   if (is_gtalk)
     {
+      gchar *fallback_servers[] = {
+          "talkx.l.google.com",
+          "talkx.l.google.com:443,oldssl",
+          "talkx.l.google.com:80",
+          NULL};
+
       empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
         NULL, NULL);
-      /* We should not have to set the server but that may cause issue with
-       * buggy router. */
-      empathy_account_settings_set_string (settings, "server",
-          "talk.google.com");
+
+      empathy_account_settings_set_strv (settings, "fallback-servers",
+          fallback_servers);
     }
   else if (is_facebook)
     {
       empathy_account_settings_set_icon_name_async (settings, "im-facebook",
         NULL, NULL);
+
+      empathy_account_settings_set_string (settings, "server",
+          "chat.facebook.com");
     }
 
   if (priv->first_resp == RESPONSE_CREATE_ACCOUNT)
@@ -474,8 +482,7 @@ account_assistant_chooser_enter_details_filter_func (
     gboolean is_facebook,
     gpointer user_data)
 {
-  if (!tp_strdiff (protocol->name, "local-xmpp") ||
-      !tp_strdiff (protocol->name, "irc"))
+  if (!tp_strdiff (protocol->name, "local-xmpp"))
     return FALSE;
 
   return TRUE;
@@ -909,6 +916,10 @@ impl_signal_apply (GtkAssistant *assistant)
       g_object_unref (priv->settings);
       priv->settings = NULL;
     }
+  else if (current_page == PAGE_IMPORT)
+    {
+      empathy_import_widget_add_selected_accounts (priv->iw);
+    }
 }
 
 static void
@@ -926,12 +937,8 @@ impl_signal_prepare (GtkAssistant *assistant,
   gint current_idx;
 
   /* check from which page we are coming from */
-  switch (priv->current_page_id)
-    {
-      case PAGE_IMPORT:
-        empathy_import_widget_add_selected_accounts (priv->iw);
-        break;
-    }
+  if (priv->current_page_id == PAGE_IMPORT)
+    empathy_import_widget_add_selected_accounts (priv->iw);
 
   current_idx = gtk_assistant_get_current_page (assistant);
   priv->current_page_id = current_idx;
@@ -1117,7 +1124,7 @@ account_assistant_build_salut_page (EmpathyAccountAssistant *self)
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox_1, TRUE, TRUE, 0);
   gtk_widget_show (hbox_1);
 
-  w = gtk_label_new ("");
+  w = gtk_label_new (NULL);
   markup = g_strdup_printf ("%s (<span style=\"italic\">%s</span>).",
       _("Empathy can automatically discover and chat with the people "
         "connected on the same network as you. "
