@@ -51,6 +51,8 @@
 #include <libempathy-gtk/empathy-contact-list-store.h>
 #include <libempathy-gtk/empathy-contact-list-view.h>
 #include <libempathy-gtk/empathy-live-search.h>
+#include <libempathy-gtk/empathy-contact-blocking-dialog.h>
+#include <libempathy-gtk/empathy-contact-search-dialog.h>
 #include <libempathy-gtk/empathy-geometry.h>
 #include <libempathy-gtk/empathy-gtk-enum-types.h>
 #include <libempathy-gtk/empathy-individual-dialogs.h>
@@ -92,8 +94,8 @@
 #define GEOMETRY_NAME "main-window"
 
 enum {
-	PAGE_NO_MATCH = 0,
-	PAGE_CONTACT_LIST
+	PAGE_CONTACT_LIST = 0,
+	PAGE_NO_MATCH
 };
 
 G_DEFINE_TYPE (EmpathyMainWindow, empathy_main_window, GTK_TYPE_WINDOW);
@@ -120,6 +122,7 @@ struct _EmpathyMainWindowPriv {
 	GtkWidget              *presence_toolbar;
 	GtkWidget              *presence_chooser;
 	GtkWidget              *errors_vbox;
+	GtkWidget              *auth_vbox;
 	GtkWidget              *search_bar;
 	GtkWidget              *notebook;
 	GtkWidget              *no_entry_label;
@@ -140,8 +143,13 @@ struct _EmpathyMainWindowPriv {
 	GtkWidget              *edit_context_separator;
 
 	guint                   size_timeout_id;
+
+	/* reffed TpAccount* => visible GtkInfoBar* */
 	GHashTable             *errors;
 
+	/* EmpathyEvent* => visible GtkInfoBar* */
+	GHashTable             *auths;
+
 	/* stores a mapping from TpAccount to Handler ID to prevent
 	 * to listen more than once to the status-changed signal */
 	GHashTable             *status_changed_handlers;
@@ -283,6 +291,138 @@ main_window_flash_start (EmpathyMainWindow *window)
 	main_window_flash_cb (window);
 }
 
+static void
+main_window_remove_auth (EmpathyMainWindow *window,
+			 EmpathyEvent      *event)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+	GtkWidget *error_widget;
+
+	error_widget = g_hash_table_lookup (priv->auths, event);
+	if (error_widget != NULL) {
+		gtk_widget_destroy (error_widget);
+		g_hash_table_remove (priv->auths, event);
+	}
+}
+
+static void
+main_window_auth_add_clicked_cb (GtkButton         *button,
+				 EmpathyMainWindow *window)
+{
+	EmpathyEvent *event;
+
+	event = g_object_get_data (G_OBJECT (button), "event");
+
+	empathy_event_approve (event);
+
+	main_window_remove_auth (window, event);
+}
+
+static void
+main_window_auth_close_clicked_cb (GtkButton         *button,
+				   EmpathyMainWindow *window)
+{
+	EmpathyEvent *event;
+
+	event = g_object_get_data (G_OBJECT (button), "event");
+
+	empathy_event_decline (event);
+	main_window_remove_auth (window, event);
+}
+
+static void
+main_window_auth_display (EmpathyMainWindow *window,
+			  EmpathyEvent      *event)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+	TpAccount *account = event->account;
+	GtkWidget *info_bar;
+	GtkWidget *content_area;
+	GtkWidget *image;
+	GtkWidget *label;
+	GtkWidget *add_button;
+	GtkWidget *close_button;
+	GtkWidget *action_area;
+	GtkWidget *action_table;
+	const gchar *icon_name;
+	gchar *str;
+
+	if (g_hash_table_lookup (priv->auths, event) != NULL) {
+		return;
+	}
+
+	info_bar = gtk_info_bar_new ();
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_QUESTION);
+
+	gtk_widget_set_no_show_all (info_bar, TRUE);
+	gtk_box_pack_start (GTK_BOX (priv->auth_vbox), info_bar, FALSE, TRUE, 0);
+	gtk_widget_show (info_bar);
+
+	icon_name = tp_account_get_icon_name (account);
+	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_SMALL_TOOLBAR);
+	gtk_widget_show (image);
+
+	str = g_markup_printf_escaped ("<b>%s</b>\n%s",
+				       tp_account_get_display_name (account),
+				       _("Password required"));
+
+	label = gtk_label_new (str);
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_widget_show (label);
+
+	g_free (str);
+
+	content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar));
+	gtk_box_pack_start (GTK_BOX (content_area), image, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (content_area), label, TRUE, TRUE, 0);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
+	add_button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (add_button), image);
+	gtk_widget_set_tooltip_text (add_button, _("Provide Password"));
+	gtk_widget_show (add_button);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON);
+	close_button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (close_button), image);
+	gtk_widget_set_tooltip_text (close_button, _("Disconnect"));
+	gtk_widget_show (close_button);
+
+	action_table = gtk_table_new (1, 2, FALSE);
+	gtk_table_set_col_spacings (GTK_TABLE (action_table), 6);
+	gtk_widget_show (action_table);
+
+	action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
+	gtk_box_pack_start (GTK_BOX (action_area), action_table, FALSE, FALSE, 0);
+
+	gtk_table_attach (GTK_TABLE (action_table), add_button, 0, 1, 0, 1,
+			  (GtkAttachOptions) (GTK_SHRINK),
+			  (GtkAttachOptions) (GTK_SHRINK), 0, 0);
+	gtk_table_attach (GTK_TABLE (action_table), close_button, 1, 2, 0, 1,
+			  (GtkAttachOptions) (GTK_SHRINK),
+			  (GtkAttachOptions) (GTK_SHRINK), 0, 0);
+
+	g_object_set_data_full (G_OBJECT (info_bar),
+				"event", event, NULL);
+	g_object_set_data_full (G_OBJECT (add_button),
+				"event", event, NULL);
+	g_object_set_data_full (G_OBJECT (close_button),
+				"event", event, NULL);
+
+	g_signal_connect (add_button, "clicked",
+			  G_CALLBACK (main_window_auth_add_clicked_cb),
+			  window);
+	g_signal_connect (close_button, "clicked",
+			  G_CALLBACK (main_window_auth_close_clicked_cb),
+			  window);
+
+	gtk_widget_show (priv->auth_vbox);
+
+	g_hash_table_insert (priv->auths, event, info_bar);
+}
+
 static void
 main_window_event_added_cb (EmpathyEventManager *manager,
 			    EmpathyEvent        *event,
@@ -290,6 +430,8 @@ main_window_event_added_cb (EmpathyEventManager *manager,
 {
 	if (event->contact) {
 		main_window_flash_start (window);
+	} else if (event->type == EMPATHY_EVENT_TYPE_AUTH) {
+		main_window_auth_display (window, event);
 	}
 }
 
@@ -301,6 +443,11 @@ main_window_event_removed_cb (EmpathyEventManager *manager,
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
 	FlashForeachData data;
 
+	if (event->type == EMPATHY_EVENT_TYPE_AUTH) {
+		main_window_remove_auth (window, event);
+		return;
+	}
+
 	if (!event->contact) {
 		return;
 	}
@@ -385,6 +532,9 @@ main_window_row_deleted_cb (GtkTreeModel      *model,
 			gtk_label_set_markup (GTK_LABEL (priv->no_entry_label), tmp);
 			g_free (tmp);
 
+			gtk_label_set_ellipsize (GTK_LABEL (priv->no_entry_label),
+				PANGO_ELLIPSIZE_END);
+
 			gtk_notebook_set_current_page (
 					GTK_NOTEBOOK (priv->notebook), PAGE_NO_MATCH);
 		}
@@ -588,6 +738,7 @@ main_window_error_display (EmpathyMainWindow *window,
 			  G_CALLBACK (main_window_error_retry_clicked_cb),
 			  window);
 
+	gtk_widget_set_tooltip_text (priv->errors_vbox, error_message);
 	gtk_widget_show (priv->errors_vbox);
 
 	g_hash_table_insert (priv->errors, g_object_ref (account), info_bar);
@@ -693,6 +844,7 @@ empathy_main_window_finalize (GObject *window)
 	g_object_unref (priv->account_manager);
 	g_object_unref (priv->individual_store);
 	g_hash_table_destroy (priv->errors);
+	g_hash_table_destroy (priv->auths);
 
 	/* disconnect all handlers of status-changed signal */
 	g_hash_table_iter_init (&iter, priv->status_changed_handlers);
@@ -770,6 +922,15 @@ main_window_chat_add_contact_cb (GtkAction         *action,
 	empathy_new_individual_dialog_show (GTK_WINDOW (window));
 }
 
+static void
+main_window_chat_search_contacts_cb (GtkAction         *action,
+				     EmpathyMainWindow *window)
+{
+	GtkWidget *dialog = empathy_contact_search_dialog_new (
+			GTK_WINDOW (window));
+	gtk_widget_show (dialog);
+}
+
 static void
 main_window_view_show_ft_manager (GtkAction         *action,
 				  EmpathyMainWindow *window)
@@ -1275,6 +1436,18 @@ main_window_edit_personal_information_cb (GtkAction         *action,
 	empathy_contact_personal_dialog_show (GTK_WINDOW (window));
 }
 
+static void
+main_window_edit_blocked_contacts_cb (GtkAction         *action,
+		                      EmpathyMainWindow *window)
+{
+	GtkWidget *dialog;
+
+	dialog = empathy_contact_blocking_dialog_new (GTK_WINDOW (window));
+	gtk_widget_show (dialog);
+	g_signal_connect (dialog, "response",
+			G_CALLBACK (gtk_widget_destroy), NULL);
+}
+
 static void
 main_window_edit_preferences_cb (GtkAction         *action,
 				 EmpathyMainWindow *window)
@@ -1426,7 +1599,8 @@ main_window_connection_items_setup (EmpathyMainWindow *window,
 		"chat_new_message",
 		"chat_new_call",
 		"chat_add_contact",
-		"edit_personal_information"
+		"edit_personal_information",
+		"edit_blocked_contacts"
 	};
 
 	for (i = 0, list = NULL; i < G_N_ELEMENTS (actions_connected); i++) {
@@ -1560,6 +1734,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	gui = empathy_builder_get_file (filename,
 				       "main_vbox", &priv->main_vbox,
 				       "errors_vbox", &priv->errors_vbox,
+				       "auth_vbox", &priv->auth_vbox,
 				       "ui_manager", &priv->ui_manager,
 				       "view_show_offline", &show_offline_widget,
 				       "view_show_protocols", &priv->show_protocols,
@@ -1593,6 +1768,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 			      "room_join_favorites", "activate", main_window_room_join_favorites_cb,
 			      "room_manage_favorites", "activate", main_window_room_manage_favorites_cb,
 			      "chat_add_contact", "activate", main_window_chat_add_contact_cb,
+			      "chat_search_contacts", "activate", main_window_chat_search_contacts_cb,
 			      "view_show_ft_manager", "activate", main_window_view_show_ft_manager,
 			      "view_show_offline", "toggled", main_window_view_show_offline_cb,
 			      "view_show_protocols", "toggled", main_window_view_show_protocols_cb,
@@ -1602,6 +1778,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 			      "edit", "activate", main_window_edit_cb,
 			      "edit_accounts", "activate", main_window_edit_accounts_cb,
 			      "edit_personal_information", "activate", main_window_edit_personal_information_cb,
+			      "edit_blocked_contacts", "activate", main_window_edit_blocked_contacts_cb,
 			      "edit_preferences", "activate", main_window_edit_preferences_cb,
 			      "help_about", "activate", main_window_help_about_cb,
 			      "help_debug", "activate", main_window_help_debug_cb,
@@ -1628,6 +1805,8 @@ empathy_main_window_init (EmpathyMainWindow *window)
 					      g_object_unref,
 					      NULL);
 
+	priv->auths = g_hash_table_new (NULL, NULL);
+
 	priv->status_changed_handlers = g_hash_table_new_full (g_direct_hash,
 							       g_direct_equal,
 							       NULL,
